@@ -35,6 +35,18 @@ $ python setup.py help
      libraries=["milter","smutil","resolv"]
 
  * $Log: miltermodule.c,v $
+ * Revision 1.15  2008/12/13 20:29:56  customdesigned
+ * Split off milter applications.
+ *
+ * Revision 1.14  2008/12/04 19:43:00  customdesigned
+ * Doc updates.
+ *
+ * Revision 1.13  2008/11/23 03:06:47  customdesigned
+ * Milter support for chgfrom.
+ *
+ * Revision 1.12  2008/11/21 20:42:52  customdesigned
+ * Support smfi_chgfrom and smfi_addrcpt_par.
+ *
  * Revision 1.11  2007/09/25 02:26:29  customdesigned
  * Update license.
  *
@@ -182,10 +194,10 @@ $ python setup.py help
 #endif
 #define _FFR_MULTILINE (MAX_ML_REPLY > 1)
 
-#include <pthread.h>
-#include <netinet/in.h>
-#include <Python.h>
-#include <libmilter/mfapi.h>
+//#include <pthread.h>	// shouldn't be needed - use Python API
+#include <Python.h>		// Python C API
+#include <libmilter/mfapi.h>	// libmilter API
+#include <netinet/in.h>		// socket API
 
 
 /* See if we have IPv4 and/or IPv6 support in this OS and in
@@ -252,10 +264,10 @@ typedef struct {
   PyThreadState *t;	/* python thread state */
 } milter_ContextObject;
 
-/* Return a borrowed reference to the python Context.  Create a
-   new Context if needed.  The new Python Context is owned by
-   the SMFICTX. The python interpreter is locked on successful
-   return, otherwise not. */
+/* Return a borrowed reference to the python Context.  Called by callbacks
+   invoked by libmilter.  Create a new Context if needed.  The new
+   Python Context is owned by the SMFICTX. The python interpreter is locked on
+   successful return, otherwise not. */
 static milter_ContextObject *
 _get_context(SMFICTX *ctx) {
   milter_ContextObject *self = smfi_getpriv(ctx);
@@ -289,7 +301,8 @@ _get_context(SMFICTX *ctx) {
   return self;
 }
 
-/* Find the SMFICTX from a Python Context. The interpreter must be locked. */
+/* Find the SMFICTX from a Python Context.  Called by context methods invoked
+   from python.  The interpreter must be locked. */ 
 static SMFICTX *
 _find_context(PyObject *c) {
   SMFICTX *ctx = NULL;
@@ -342,6 +355,7 @@ static char milter_set_flags__doc__[] =
 Set flags for filter capabilities; OR of one or more of:\n\
 ADDHDRS - filter may add headers\n\
 CHGBODY - filter may replace body\n\
+CHGFROM - filter may replace body\n\
 ADDRCPT - filter may add recipients\n\
 DELRCPT - filter may delete recipients\n\
 CHGHDRS - filter may change/delete headers";
@@ -1016,6 +1030,30 @@ milter_addheader(PyObject *self, PyObject *args) {
 #endif
 }
 
+#ifdef SMFIF_CHGFROM
+static char milter_chgfrom__doc__[] =
+"chgfrom(sender,params) -> None\n\
+Change the envelope sender (MAIL From) of the current message.\n\
+A filter which calls smfi_chgfrom must have set the CHGFROM flag\n\
+in set_flags() before calling register.\n\
+This function can only be called from the EOM callback.";
+static PyObject *
+milter_chgfrom(PyObject *self, PyObject *args) {
+  char *sender;
+  char *params;
+  SMFICTX *ctx;
+  PyThreadState *t;
+  
+  if (!PyArg_ParseTuple(args, "s|z:chgfrom", &sender, &params))
+    return NULL;
+  ctx = _find_context(self);
+  if (ctx == NULL) return NULL;
+  t = PyEval_SaveThread();
+  return _thread_return(t,smfi_chgfrom(ctx, sender, params),
+			 "cannot change sender");
+}
+#endif
+
 static char milter_chgheader__doc__[] =
 "chgheader(field, int, value) -> None\n\
 Change/delete a header in the message. \n\
@@ -1045,22 +1083,33 @@ milter_chgheader(PyObject *self, PyObject *args) {
 }
 
 static char milter_addrcpt__doc__[] =
-"addrcpt(string) -> None\n\
+"addrcpt(string,params=None) -> None\n\
 Add a recipient to the envelope.  It must be in the same format\n\
 as is passed to the envrcpt callback in the first tuple element.\n\
+If params is used, you must pass ADDRCPT_PAR to set_flags().\n\
 This function can only be called from the EOM callback.";
 
 static PyObject *
 milter_addrcpt(PyObject *self, PyObject *args) {
   char *rcpt;
+  char *params = 0;
   SMFICTX *ctx;
   PyThreadState *t;
+  int rc;
   
-  if (!PyArg_ParseTuple(args, "s:addrcpt", &rcpt)) return NULL;
+  if (!PyArg_ParseTuple(args, "s|z:addrcpt", &rcpt)) return NULL;
   ctx = _find_context(self);
   if (ctx == NULL) return NULL;
   t = PyEval_SaveThread();
-  return _thread_return(t,smfi_addrcpt(ctx, rcpt), "cannot add recipient");
+  if (params)
+#ifdef SMFIF_ADDRCPT_PAR
+    rc = smfi_addrcpt_par(ctx,rcpt,params);
+#else
+    rc = MI_FAILURE;
+#endif
+  else
+    rc = smfi_addrcpt(ctx,rcpt);
+  return _thread_return(t,rc, "cannot add recipient");
 }
 
 static char milter_delrcpt__doc__[] =
@@ -1201,6 +1250,9 @@ static PyMethodDef context_methods[] = {
 #endif
 #ifdef SMFIR_PROGRESS
   { "progress",  milter_progress,  METH_VARARGS, milter_progress__doc__},
+#endif
+#ifdef SMFIF_CHGFROM
+  { "chgfrom",  milter_chgfrom,  METH_VARARGS, milter_chgfrom__doc__},
 #endif
   { NULL, NULL }
 };
@@ -1302,6 +1354,9 @@ initmilter(void) {
    setitem(d,"CHGBODY",  SMFIF_CHGBODY);
    setitem(d,"MODBODY",  SMFIF_MODBODY);
    setitem(d,"ADDRCPT",  SMFIF_ADDRCPT);
+#ifdef SMFIF_ADDRCPT_PAR
+   setitem(d,"ADDRCPT_PAR",  SMFIF_ADDRCPT_PAR);
+#endif
    setitem(d,"DELRCPT",  SMFIF_DELRCPT);
    setitem(d,"CHGHDRS",  SMFIF_CHGHDRS);
    setitem(d,"V1_ACTS",  SMFI_V1_ACTS);
@@ -1309,6 +1364,9 @@ initmilter(void) {
    setitem(d,"CURR_ACTS",  SMFI_CURR_ACTS);
 #ifdef SMFIF_QUARANTINE
    setitem(d,"QUARANTINE",SMFIF_QUARANTINE);
+#endif
+#ifdef SMFIF_CHGFROM
+   setitem(d,"CHGFROM",SMFIF_CHGFROM);
 #endif
    setitem(d,"CONTINUE",  SMFIS_CONTINUE);
    setitem(d,"REJECT",  SMFIS_REJECT);
